@@ -13,7 +13,7 @@ from .. import checks as checks_module
 from .. import models, runner
 from ..fixups import ALL_FIXUPS
 from ..hooks import GLOBAL_HOOK_DISPATCHER, HookContext, HookDispatcher, HookScope
-from ..runner import events
+from ..runner import DEFAULT_STATEFUL_RECURSION_LIMIT, events
 from ..runner.targeted import DEFAULT_TARGETS_NAMES, Target
 from ..types import Filter
 from ..utils import WSGIResponse
@@ -148,7 +148,7 @@ def schemathesis(pre_run: Optional[str] = None) -> None:
     type=str,
     callback=callbacks.validate_base_url,
 )
-@click.option("--app", help="WSGI application to test.", type=str, callback=callbacks.validate_app)
+@click.option("--app", help="WSGI/ASGI application to test.", type=str, callback=callbacks.validate_app)
 @click.option(
     "--request-timeout",
     help="Timeout in milliseconds for network requests during the test run.",
@@ -170,6 +170,13 @@ def schemathesis(pre_run: Optional[str] = None) -> None:
     multiple=True,
     type=click.Choice(list(ALL_FIXUPS) + ["all"]),
 )
+@click.option("--stateful", help="Utilize stateful testing capabilities.", type=click.Choice(["links"]))
+@click.option(
+    "--stateful-recursion-limit",
+    help="Limit recursion depth for stateful testing.",
+    default=DEFAULT_STATEFUL_RECURSION_LIMIT,
+    type=click.IntRange(1, 100),
+)
 @click.option(
     "--hypothesis-deadline",
     help="Duration in milliseconds that each individual example with a test is not allowed to exceed.",
@@ -198,6 +205,7 @@ def schemathesis(pre_run: Optional[str] = None) -> None:
     type=click.Choice([item.name for item in hypothesis.Verbosity]),
     callback=callbacks.convert_verbosity,
 )
+@click.option("--verbosity", "-v", help="Reduce verbosity of error output.", count=True)
 def run(  # pylint: disable=too-many-arguments
     schema: str,
     auth: Optional[Tuple[str, str]],
@@ -219,6 +227,8 @@ def run(  # pylint: disable=too-many-arguments
     show_errors_tracebacks: bool = False,
     store_network_log: Optional[click.utils.LazyFile] = None,
     fixups: Tuple[str] = (),  # type: ignore
+    stateful: Optional[str] = None,
+    stateful_recursion_limit: int = DEFAULT_STATEFUL_RECURSION_LIMIT,
     hypothesis_deadline: Optional[Union[int, NotSet]] = None,
     hypothesis_derandomize: Optional[bool] = None,
     hypothesis_max_examples: Optional[int] = None,
@@ -227,6 +237,7 @@ def run(  # pylint: disable=too-many-arguments
     hypothesis_suppress_health_check: Optional[List[hypothesis.HealthCheck]] = None,
     hypothesis_seed: Optional[int] = None,
     hypothesis_verbosity: Optional[hypothesis.Verbosity] = None,
+    verbosity: int = 0,
 ) -> None:
     """Perform schemathesis test against an API specified by SCHEMA.
 
@@ -260,6 +271,8 @@ def run(  # pylint: disable=too-many-arguments
         workers_num=workers_num,
         validate_schema=validate_schema,
         fixups=fixups,
+        stateful=stateful,
+        stateful_recursion_limit=stateful_recursion_limit,
         hypothesis_deadline=hypothesis_deadline,
         hypothesis_derandomize=hypothesis_derandomize,
         hypothesis_max_examples=hypothesis_max_examples,
@@ -268,7 +281,7 @@ def run(  # pylint: disable=too-many-arguments
         hypothesis_suppress_health_check=hypothesis_suppress_health_check,
         hypothesis_verbosity=hypothesis_verbosity,
     )
-    execute(prepared_runner, workers_num, show_errors_tracebacks, store_network_log, junit_xml)
+    execute(prepared_runner, workers_num, show_errors_tracebacks, store_network_log, junit_xml, verbosity)
 
 
 def get_output_handler(workers_num: int) -> EventHandler:
@@ -298,12 +311,13 @@ class OutputStyle(Enum):
     short = output.short.ShortOutputStyleHandler
 
 
-def execute(
+def execute(  # pylint: disable=too-many-arguments
     prepared_runner: Generator[events.ExecutionEvent, None, None],
     workers_num: int,
     show_errors_tracebacks: bool,
     store_network_log: Optional[click.utils.LazyFile],
     junit_xml: Optional[click.utils.LazyFile],
+    verbosity: int,
 ) -> None:
     """Execute a prepared runner by drawing events from it and passing to a proper handler."""
     handlers: List[EventHandler] = []
@@ -318,6 +332,7 @@ def execute(
         show_errors_tracebacks=show_errors_tracebacks,
         cassette_file_name=store_network_log.name if store_network_log is not None else None,
         junit_xml_file=junit_xml.name if junit_xml is not None else None,
+        verbosity=verbosity,
     )
     GLOBAL_HOOK_DISPATCHER.dispatch("after_init_cli_run_handlers", HookContext(), handlers, execution_context)
     try:
