@@ -31,7 +31,7 @@ class SchemathesisCase(PyCollector):
         super().__init__(*args, **kwargs)
 
     def _get_test_name(self, endpoint: Endpoint) -> str:
-        return f"{self.name}[{endpoint.method}:{endpoint.path}]"
+        return f"{self.name}[{endpoint.method}:{endpoint.full_path}]"
 
     def _gen_items(self, endpoint: Endpoint) -> Generator[Function, None, None]:
         """Generate all items for the given endpoint.
@@ -91,8 +91,9 @@ class SchemathesisCase(PyCollector):
     def _make_test(self, endpoint: Endpoint) -> Callable:
         try:
             return create_test(endpoint, self.test_function)
-        except InvalidSchema:
-            return lambda: pytest.fail("Invalid schema for endpoint")
+        except InvalidSchema as exc:
+            message = exc.args[0]
+            return lambda: pytest.fail(message)
 
     def collect(self) -> List[Function]:  # type: ignore
         """Generate different test items for all endpoints available in the given schema."""
