@@ -1,5 +1,4 @@
 import inspect
-import warnings
 from collections import defaultdict
 from enum import Enum, unique
 from typing import Any, Callable, DefaultDict, Dict, List, Optional, Union, cast
@@ -8,17 +7,8 @@ import attr
 from hypothesis import strategies as st
 
 from .models import Case, Endpoint
-from .types import GenericTest, Hook
-
-
-def warn_deprecated_hook(hook: Hook) -> None:
-    if "context" not in inspect.signature(hook).parameters:
-        warnings.warn(
-            DeprecationWarning(
-                "Hook functions that do not accept `context` argument are deprecated and "
-                "support will be removed in Schemathesis 2.0."
-            )
-        )
+from .types import GenericTest
+from .utils import GenericResponse
 
 
 class HookLocation(Enum):
@@ -81,28 +71,33 @@ class HookDispatcher:
 
             def decorator(func: Callable) -> Callable:
                 hook_name = cast(str, hook)
-                return self.register_hook_with_name(hook_name, func)
+                return self.register_hook_with_name(func, hook_name)
 
             return decorator
-        return self.register_hook_with_name(hook.__name__, hook)
+        return self.register_hook_with_name(hook, hook.__name__)
 
-    def apply(self, name: str, hook: Callable, skip_validation: bool = False) -> Callable[[Callable], Callable]:
+    def apply(self, hook: Callable, *, name: Optional[str] = None) -> Callable[[Callable], Callable]:
         """Register hook to run only on one test function.
 
         Example:
-            def hook(strategy, context):
+            def before_generate_query(strategy, context):
                 ...
 
-            @schema.hooks.apply("before_generate_query", hook)
+            @schema.hooks.apply(before_generate_query)
             @schema.parametrize()
             def test_api(case):
                 ...
 
         """
 
+        if name is None:
+            hook_name = hook.__name__
+        else:
+            hook_name = name
+
         def decorator(func: GenericTest) -> GenericTest:
             dispatcher = self.add_dispatcher(func)
-            dispatcher.register_hook_with_name(name, hook, skip_validation)
+            dispatcher.register_hook_with_name(hook, hook_name)
             return func
 
         return decorator
@@ -114,14 +109,9 @@ class HookDispatcher:
             func._schemathesis_hooks = cls(scope=HookScope.TEST)  # type: ignore
         return func._schemathesis_hooks  # type: ignore
 
-    def register_hook_with_name(self, name: str, hook: Callable, skip_validation: bool = False) -> Callable:
-        """A helper for hooks registration.
-
-        Besides its use in this class internally it is used to keep backward compatibility with the old hooks system.
-        """
-        # Validation is skipped only for backward compatibility with the old hooks system
-        if not skip_validation:
-            self._validate_hook(name, hook)
+    def register_hook_with_name(self, hook: Callable, name: str) -> Callable:
+        """A helper for hooks registration."""
+        self._validate_hook(name, hook)
         self._hooks[name].append(hook)
         return hook
 
@@ -230,28 +220,17 @@ def before_add_examples(context: HookContext, examples: List[Case]) -> None:
     """
 
 
+@HookDispatcher.register_spec([HookScope.GLOBAL])
+def add_case(context: HookContext, case: Case, response: GenericResponse) -> Optional[Case]:
+    """Creates an additional test per endpoint. If this hook returns None, no additional test created.
+
+    Called with a copy of the original case object and the server's response to the original case.
+    """
+
+
 GLOBAL_HOOK_DISPATCHER = HookDispatcher(scope=HookScope.GLOBAL)
 dispatch = GLOBAL_HOOK_DISPATCHER.dispatch
 get_all_by_name = GLOBAL_HOOK_DISPATCHER.get_all_by_name
+register = GLOBAL_HOOK_DISPATCHER.register
 unregister = GLOBAL_HOOK_DISPATCHER.unregister
 unregister_all = GLOBAL_HOOK_DISPATCHER.unregister_all
-
-
-def register(*args: Union[str, Callable]) -> Callable:
-    # This code suppose to support backward compatibility with the old hook system.
-    # In Schemathesis 2.0 this function can be replaced with `register = GLOBAL_HOOK_DISPATCHER.register`
-    if len(args) == 1:
-        return GLOBAL_HOOK_DISPATCHER.register(args[0])
-    if len(args) == 2:
-        warnings.warn(
-            "Calling `schemathesis.register` with two arguments is deprecated, use it as a decorator instead.",
-            DeprecationWarning,
-        )
-        place, hook = args
-        hook = cast(Callable, hook)
-        warn_deprecated_hook(hook)
-        if place not in HookLocation.__members__:
-            raise KeyError(place)
-        return GLOBAL_HOOK_DISPATCHER.register_hook_with_name(f"before_generate_{place}", hook, skip_validation=True)
-    # This approach is quite naive, but it should be enough for the common use case
-    raise TypeError("Invalid number of arguments. Please, use `register` as a decorator.")
