@@ -4,15 +4,17 @@ import datetime
 import http
 from collections import Counter
 from contextlib import contextmanager
+from copy import deepcopy
 from enum import IntEnum
 from logging import LogRecord
-from typing import TYPE_CHECKING, Any, Callable, Dict, Generator, Iterator, List, Optional, Tuple, Union, cast
-from urllib.parse import urljoin
+from typing import TYPE_CHECKING, Any, Callable, Dict, Generator, Iterator, List, Optional, Sequence, Tuple, Union, cast
+from urllib.parse import urljoin, urlsplit, urlunsplit
 
 import attr
 import requests
 import werkzeug
 from hypothesis.strategies import SearchStrategy
+from starlette.testclient import TestClient as ASGIClient
 
 from .checks import ALL_CHECKS
 from .exceptions import InvalidSchema
@@ -20,8 +22,9 @@ from .types import Body, Cookies, FormData, Headers, PathParameters, Query
 from .utils import GenericResponse, WSGIResponse
 
 if TYPE_CHECKING:
-    from .schemas import BaseSchema
     from .hooks import HookDispatcher
+    from .schemas import BaseSchema
+    from .stateful import StatefulTest
 
 
 @attr.s(slots=True)  # pragma: no mutate
@@ -40,6 +43,10 @@ class Case:
     def path(self) -> str:
         return self.endpoint.path
 
+    @property
+    def full_path(self) -> str:
+        return self.endpoint.full_path
+
     @property
     def method(self) -> str:
         return self.endpoint.method
@@ -57,12 +64,22 @@ class Case:
         # pylint: disable=not-a-mapping
         try:
             return self.path.format(**self.path_parameters or {})
-        except KeyError:
-            raise InvalidSchema("Missing required property `required: true`")
+        except KeyError as variable:
+            raise InvalidSchema(
+                f"Missing path parameter {variable}. It either not defined or has no `required: true` in its definition"
+            )
+
+    def get_full_base_url(self) -> Optional[str]:
+        """Create a full base url, adding "localhost" for WSGI apps."""
+        parts = urlsplit(self.base_url)
+        if not parts.hostname:
+            path = cast(str, parts.path or "")
+            return urlunsplit(("http", "localhost", path or "", "", ""))
+        return self.base_url
 
     def get_code_to_reproduce(self, headers: Optional[Dict[str, Any]] = None) -> str:
         """Construct a Python code to reproduce this case with `requests`."""
-        base_url = self.base_url or "http://localhost"
+        base_url = self.get_full_base_url()
         kwargs = self.as_requests_kwargs(base_url)
         if headers:
             final_headers = kwargs["headers"] or {}
@@ -103,7 +120,8 @@ class Case:
         # Form data and body are mutually exclusive
         extra: Dict[str, Optional[Body]]
         if self.form_data:
-            extra = {"files": self.form_data}
+            files, data = self.endpoint.prepare_multipart(self.form_data)
+            extra = {"files": files, "data": data}
         elif is_multipart(self.body):
             extra = {"data": self.body}
         else:
@@ -131,7 +149,6 @@ class Case:
         else:
             close_session = False
 
-        base_url = self._get_base_url(base_url)
         data = self.as_requests_kwargs(base_url)
         if headers is not None:
             data["headers"] = {**(data["headers"] or {}), **headers}
@@ -150,14 +167,15 @@ class Case:
         if self.form_data:
             extra = {"data": self.form_data}
             final_headers = final_headers or {}
-            final_headers.setdefault("Content-Type", "multipart/form-data")
+            if "multipart/form-data" in self.endpoint.get_request_payload_content_types():
+                final_headers.setdefault("Content-Type", "multipart/form-data")
         elif is_multipart(self.body):
             extra = {"data": self.body}
         else:
             extra = {"json": self.body}
         return {
             "method": self.method,
-            "path": self.formatted_path,
+            "path": self.endpoint.schema.get_full_path(self.formatted_path),
             "headers": final_headers,
             "query_string": self.query,
             **extra,
@@ -175,6 +193,23 @@ class Case:
         with cookie_handler(client, self.cookies):
             return client.open(**data, **kwargs)
 
+    def call_asgi(
+        self,
+        app: Any = None,
+        base_url: Optional[str] = "http://testserver",
+        headers: Optional[Dict[str, str]] = None,
+        **kwargs: Any,
+    ) -> requests.Response:
+        application = app or self.app
+        if application is None:
+            raise RuntimeError(
+                "ASGI application instance is required. "
+                "Please, set `app` argument in the schema constructor or pass it to `call_asgi`"
+            )
+        client = ASGIClient(application)
+
+        return self.call(base_url=base_url, session=client, headers=headers, **kwargs)
+
     def validate_response(
         self,
         response: Union[requests.Response, WSGIResponse],
@@ -189,6 +224,25 @@ class Case:
         if errors:
             raise AssertionError(*errors)
 
+    def get_full_url(self) -> str:
+        """Make a full URL to the current endpoint, including query parameters."""
+        base_url = self.base_url or "http://localhost"
+        kwargs = self.as_requests_kwargs(base_url)
+        request = requests.Request(**kwargs)
+        prepared = requests.Session().prepare_request(request)  # type: ignore
+        return prepared.url
+
+    def partial_deepcopy(self) -> "Case":
+        return self.__class__(
+            endpoint=self.endpoint.partial_deepcopy(),
+            path_parameters=deepcopy(self.path_parameters),
+            headers=deepcopy(self.headers),
+            cookies=deepcopy(self.cookies),
+            query=deepcopy(self.query),
+            body=deepcopy(self.body),
+            form_data=deepcopy(self.form_data),
+        )
+
 
 def is_multipart(item: Optional[Body]) -> bool:
     """A poor detection if the body should be a multipart request.
@@ -238,6 +292,7 @@ class EndpointDefinition:
     """
 
     raw: Dict[str, Any] = attr.ib()  # pragma: no mutate
+    resolved: Dict[str, Any] = attr.ib()  # pragma: no mutate
     scope: str = attr.ib()  # pragma: no mutate
 
 
@@ -245,6 +300,9 @@ class EndpointDefinition:
 class Endpoint:
     """A container that could be used for test cases generation."""
 
+    # `path` does not contain `basePath`
+    # Example <scheme>://<host>/<basePath>/users - "/users" is path
+    # https://swagger.io/docs/specification/2-0/api-host-and-base-path/
     path: str = attr.ib()  # pragma: no mutate
     method: str = attr.ib()  # pragma: no mutate
     definition: EndpointDefinition = attr.ib()  # pragma: no mutate
@@ -258,11 +316,52 @@ class Endpoint:
     body: Optional[Body] = attr.ib(default=None)  # pragma: no mutate
     form_data: Optional[FormData] = attr.ib(default=None)  # pragma: no mutate
 
+    @property
+    def full_path(self) -> str:
+        return self.schema.get_full_path(self.path)
+
     def as_strategy(self, hooks: Optional["HookDispatcher"] = None) -> SearchStrategy:
         from ._hypothesis import get_case_strategy  # pylint: disable=import-outside-toplevel
 
         return get_case_strategy(self, hooks)
 
+    def get_strategies_from_examples(self) -> List[SearchStrategy[Case]]:
+        """Get examples from endpoint."""
+        return self.schema.get_strategies_from_examples(self)
+
+    def get_stateful_tests(self, response: GenericResponse, stateful: Optional[str]) -> Sequence["StatefulTest"]:
+        return self.schema.get_stateful_tests(response, self, stateful)
+
+    def get_hypothesis_conversions(self, location: str) -> Optional[Callable]:
+        definitions = [item for item in self.definition.resolved.get("parameters", []) if item["in"] == location]
+        if definitions:
+            return self.schema.get_hypothesis_conversion(definitions)
+        return None
+
+    def prepare_multipart(self, form_data: Optional[FormData]) -> Tuple[Optional[List], Optional[Dict[str, Any]]]:
+        if not form_data:
+            return None, None
+        return self.schema.prepare_multipart(form_data, self)
+
+    def get_request_payload_content_types(self) -> List[str]:
+        return self.schema.get_request_payload_content_types(self)
+
+    def partial_deepcopy(self) -> "Endpoint":
+        return self.__class__(
+            path=self.path,  # string, immutable
+            method=self.method,  # string, immutable
+            definition=deepcopy(self.definition),
+            schema=self.schema.clone(),  # shallow copy
+            app=self.app,  # not deepcopyable
+            base_url=self.base_url,  # string, immutable
+            path_parameters=deepcopy(self.path_parameters),
+            headers=deepcopy(self.path_parameters),
+            cookies=deepcopy(self.cookies),
+            query=deepcopy(self.query),
+            body=deepcopy(self.body),
+            form_data=deepcopy(self.form_data),
+        )
+
 
 class Status(IntEnum):
     """Status of an action or multiple actions."""
@@ -294,7 +393,7 @@ class Request:
     @classmethod
     def from_case(cls, case: Case, session: requests.Session) -> "Request":
         """Create a new `Request` instance from `Case`."""
-        base_url = case.base_url or "http://localhost"
+        base_url = case.get_full_base_url()
         kwargs = case.as_requests_kwargs(base_url)
         request = requests.Request(**kwargs)
         prepared = session.prepare_request(request)  # type: ignore
