@@ -2,6 +2,7 @@ from typing import Any, Callable, Dict, Generator, Iterable, List, Optional, Tup
 from urllib.parse import urlparse
 
 import hypothesis.errors
+from starlette.applications import Starlette
 
 from .. import fixups as _fixups
 from .. import loaders
@@ -9,9 +10,18 @@ from ..checks import DEFAULT_CHECKS
 from ..models import CheckFunction
 from ..schemas import BaseSchema
 from ..types import Filter, NotSet, RawAuth
-from ..utils import dict_not_none_values, dict_true_values, file_exists, get_base_url, get_requests_auth, import_app
+from ..utils import dict_not_none_values, dict_true_values, file_exists, get_requests_auth, import_app
 from . import events
-from .impl import BaseRunner, SingleThreadRunner, SingleThreadWSGIRunner, ThreadPoolRunner, ThreadPoolWSGIRunner
+from .impl import (
+    DEFAULT_STATEFUL_RECURSION_LIMIT,
+    BaseRunner,
+    SingleThreadASGIRunner,
+    SingleThreadRunner,
+    SingleThreadWSGIRunner,
+    ThreadPoolASGIRunner,
+    ThreadPoolRunner,
+    ThreadPoolWSGIRunner,
+)
 from .targeted import DEFAULT_TARGETS, Target
 
 
@@ -26,6 +36,8 @@ def prepare(  # pylint: disable=too-many-arguments
     exit_first: bool = False,
     store_interactions: bool = False,
     fixups: Iterable[str] = (),
+    stateful: Optional[str] = None,
+    stateful_recursion_limit: int = DEFAULT_STATEFUL_RECURSION_LIMIT,
     # Schema loading
     loader: Callable = loaders.from_uri,
     base_url: Optional[str] = None,
@@ -87,6 +99,8 @@ def prepare(  # pylint: disable=too-many-arguments
         request_timeout=request_timeout,
         store_interactions=store_interactions,
         fixups=fixups,
+        stateful=stateful,
+        stateful_recursion_limit=stateful_recursion_limit,
     )
 
 
@@ -132,6 +146,8 @@ def execute_from_schema(
     exit_first: bool = False,
     store_interactions: bool = False,
     fixups: Iterable[str] = (),
+    stateful: Optional[str] = None,
+    stateful_recursion_limit: int = DEFAULT_STATEFUL_RECURSION_LIMIT,
 ) -> Generator[events.ExecutionEvent, None, None]:
     """Execute tests for the given schema.
 
@@ -164,21 +180,7 @@ def execute_from_schema(
 
         runner: BaseRunner
         if workers_num > 1:
-            if schema.app:
-                runner = ThreadPoolWSGIRunner(
-                    schema=schema,
-                    checks=checks,
-                    targets=targets,
-                    hypothesis_settings=hypothesis_options,
-                    auth=auth,
-                    auth_type=auth_type,
-                    headers=headers,
-                    seed=seed,
-                    workers_num=workers_num,
-                    exit_first=exit_first,
-                    store_interactions=store_interactions,
-                )
-            else:
+            if not schema.app:
                 runner = ThreadPoolRunner(
                     schema=schema,
                     checks=checks,
@@ -192,10 +194,11 @@ def execute_from_schema(
                     request_timeout=request_timeout,
                     exit_first=exit_first,
                     store_interactions=store_interactions,
+                    stateful=stateful,
+                    stateful_recursion_limit=stateful_recursion_limit,
                 )
-        else:
-            if schema.app:
-                runner = SingleThreadWSGIRunner(
+            elif isinstance(schema.app, Starlette):
+                runner = ThreadPoolASGIRunner(
                     schema=schema,
                     checks=checks,
                     targets=targets,
@@ -206,8 +209,29 @@ def execute_from_schema(
                     seed=seed,
                     exit_first=exit_first,
                     store_interactions=store_interactions,
+                    stateful=stateful,
+                    stateful_recursion_limit=stateful_recursion_limit,
                 )
+
             else:
+                runner = ThreadPoolWSGIRunner(
+                    schema=schema,
+                    checks=checks,
+                    targets=targets,
+                    hypothesis_settings=hypothesis_options,
+                    auth=auth,
+                    auth_type=auth_type,
+                    headers=headers,
+                    seed=seed,
+                    workers_num=workers_num,
+                    exit_first=exit_first,
+                    store_interactions=store_interactions,
+                    stateful=stateful,
+                    stateful_recursion_limit=stateful_recursion_limit,
+                )
+
+        else:
+            if not schema.app:
                 runner = SingleThreadRunner(
                     schema=schema,
                     checks=checks,
@@ -220,7 +244,40 @@ def execute_from_schema(
                     request_timeout=request_timeout,
                     exit_first=exit_first,
                     store_interactions=store_interactions,
+                    stateful=stateful,
+                    stateful_recursion_limit=stateful_recursion_limit,
                 )
+            elif isinstance(schema.app, Starlette):
+                runner = SingleThreadASGIRunner(
+                    schema=schema,
+                    checks=checks,
+                    targets=targets,
+                    hypothesis_settings=hypothesis_options,
+                    auth=auth,
+                    auth_type=auth_type,
+                    headers=headers,
+                    seed=seed,
+                    exit_first=exit_first,
+                    store_interactions=store_interactions,
+                    stateful=stateful,
+                    stateful_recursion_limit=stateful_recursion_limit,
+                )
+            else:
+                runner = SingleThreadWSGIRunner(
+                    schema=schema,
+                    checks=checks,
+                    targets=targets,
+                    hypothesis_settings=hypothesis_options,
+                    auth=auth,
+                    auth_type=auth_type,
+                    headers=headers,
+                    seed=seed,
+                    exit_first=exit_first,
+                    store_interactions=store_interactions,
+                    stateful=stateful,
+                    stateful_recursion_limit=stateful_recursion_limit,
+                )
+
         yield from runner.execute()
     except Exception as exc:
         yield events.InternalError.from_exc(exc)
@@ -255,11 +312,10 @@ def load_schema(
             # If `schema` is not an existing filesystem path or an URL then it is considered as an endpoint with
             # the given app
             loader = loaders.get_loader_for_app(app)
+            loader_options.update(dict_true_values(headers=headers))
         else:
             loader_options.update(dict_true_values(headers=headers, auth=auth, auth_type=auth_type))
 
-    if "base_url" not in loader_options and not isinstance(schema_uri, dict):
-        loader_options["base_url"] = get_base_url(schema_uri)
     if loader is loaders.from_uri and loader_options.get("auth"):
         loader_options["auth"] = get_requests_auth(loader_options["auth"], loader_options.pop("auth_type", None))
 
