@@ -1,13 +1,17 @@
 from pathlib import Path
-from typing import Any, Callable, Dict, List, NewType, Set, Tuple, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, NewType, Set, Tuple, Union
 
 from hypothesis.strategies import SearchStrategy
 
+if TYPE_CHECKING:
+    from .hooks import HookContext
+
 Schema = NewType("Schema", Dict[str, Any])  # pragma: no mutate
 PathLike = Union[Path, str]  # pragma: no mutate
 
 Query = Dict[str, Any]  # pragma: no mutate
-Body = Union[Dict[str, Any], bytes]  # pragma: no mutate
+# Body can be of any Python type that corresponds to JSON Schema types + `bytes`
+Body = Union[List, Dict[str, Any], str, int, float, bool, bytes]  # pragma: no mutate
 PathParameters = Dict[str, Any]  # pragma: no mutate
 Headers = Dict[str, Any]  # pragma: no mutate
 Cookies = Dict[str, Any]  # pragma: no mutate
@@ -21,6 +25,10 @@ class NotSet:
 # A filter for endpoint / method
 Filter = Union[str, List[str], Tuple[str], Set[str], NotSet]  # pragma: no mutate
 
-Hook = Callable[[SearchStrategy], SearchStrategy]  # pragma: no mutate
+Hook = Union[
+    Callable[[SearchStrategy], SearchStrategy], Callable[[SearchStrategy, "HookContext"], SearchStrategy]
+]  # pragma: no mutate
 
 RawAuth = Tuple[str, str]  # pragma: no mutate
+# Generic test with any arguments and no return
+GenericTest = Callable[..., None]  # pragma: no mutate
