@@ -3,11 +3,8 @@ import pathlib
 import re
 import sys
 import traceback
-import warnings
 from contextlib import contextmanager
-from functools import wraps
 from typing import Any, Callable, Dict, Generator, List, Optional, Set, Tuple, Type, Union, overload
-from urllib.parse import urlsplit, urlunsplit
 
 import requests
 import yaml
@@ -30,20 +27,6 @@ except ImportError:
 NOT_SET = NotSet()
 
 
-def deprecated(message: str) -> Callable:
-    """Emit a warning if the given function is used."""
-
-    def wrapper(func: Callable) -> Callable:
-        @wraps(func)  # pragma: no mutate
-        def inner(*args: Any, **kwargs: Any) -> Any:
-            warnings.warn(message, DeprecationWarning)
-            return func(*args, **kwargs)
-
-        return inner
-
-    return wrapper
-
-
 def file_exists(path: str) -> bool:
     try:
         return pathlib.Path(path).is_file()
@@ -85,12 +68,6 @@ def is_schemathesis_test(func: Callable) -> bool:
         return False
 
 
-def get_base_url(uri: str) -> str:
-    """Remove the path part off the given uri."""
-    parts = urlsplit(uri)[:2] + ("", "", "")
-    return urlunsplit(parts)
-
-
 def force_tuple(item: Filter) -> Union[List, Set, Tuple]:
     if not isinstance(item, (list, set, tuple)):
         return (item,)
@@ -169,6 +146,23 @@ def make_loader(*tags_to_remove: str) -> Type[yaml.SafeLoader]:
         key: [(tag, regexp) for tag, regexp in mapping if tag not in tags_to_remove]
         for key, mapping in cls.yaml_implicit_resolvers.copy().items()
     }
+
+    # Fix pyyaml scientific notation parse bug
+    # See PR: https://github.com/yaml/pyyaml/pull/174 for upstream fix
+    cls.add_implicit_resolver(  # type: ignore
+        "tag:yaml.org,2002:float",
+        re.compile(
+            r"""^(?:[-+]?(?:[0-9][0-9_]*)\.[0-9_]*(?:[eE][-+]?[0-9]+)?
+                       |[-+]?(?:[0-9][0-9_]*)(?:[eE][-+]?[0-9]+)
+                       |\.[0-9_]+(?:[eE][-+]?[0-9]+)?
+                       |[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+\.[0-9_]*
+                       |[-+]?\.(?:inf|Inf|INF)
+                       |\.(?:nan|NaN|NAN))$""",
+            re.X,
+        ),
+        list("-+0123456789."),
+    )
+
     return cls
 
 
